@@ -1,5 +1,5 @@
 /*
-  Copyright 2003 Savonet team
+  Copyright 2003-2006 Savonet team
 
   This file is part of Ocaml-shout.
 
@@ -24,11 +24,12 @@
  * @author Samuel Mimram
  */
 
-/* $Id: shout_stubs.c,v 1.12 2005/03/05 15:51:33 smimram Exp $ */
+/* $Id: shout_stubs.c 2338 2006-02-14 10:14:52Z smimram $ */
 
 #define CAML_NAME_SPACE
 #include <caml/alloc.h>
 #include <caml/callback.h>
+#include <caml/custom.h>
 #include <caml/fail.h>
 #include <caml/memory.h>
 #include <caml/misc.h>
@@ -37,37 +38,43 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <assert.h>
 #include <shout/shout.h>
 
+#define Shout_val(v) (*((shout_t**)Data_custom_val(v)))
 
-static shout_t* shout_of_block(value block)
+static void finalize_shout (value block)
 {
-  return (shout_t*)Field(block, 1);
+  shout_free(Shout_val(block));
 }
 
-static void finalize_shout_t (value block)
+static struct custom_operations shout_ops =
 {
-  shout_t *s = shout_of_block(block);
-  shout_free(s);
-}
+  "ocaml_shout_shout",
+  finalize_shout,
+  custom_compare_default,
+  custom_hash_default,
+  custom_serialize_default,
+  custom_deserialize_default
+};
 
-CAMLprim value ocaml_shout_init()
+CAMLprim value ocaml_shout_init(value unit)
 {
-  CAMLparam0();
+  CAMLparam1(unit);
   shout_init();
   CAMLreturn(Val_unit);
 }
 
-CAMLprim value ocaml_shout_shutdown()
+CAMLprim value ocaml_shout_shutdown(value unit)
 {
-  CAMLparam0();
+  CAMLparam1(unit);
   shout_shutdown();
   CAMLreturn(Val_unit);
 }
 
-CAMLprim value ocaml_shout_version()
+CAMLprim value ocaml_shout_version(value unit)
 {
-  CAMLparam0();
+  CAMLparam1(unit);
   CAMLlocal1(block);
   int major, minor, patch;
   const char *version = shout_version(&major, &minor, &patch);
@@ -79,23 +86,22 @@ CAMLprim value ocaml_shout_version()
   CAMLreturn(block);
 }
 
-CAMLprim value ocaml_shout_new ()
+CAMLprim value ocaml_shout_new(value unit)
 {
-  CAMLparam0();
+  CAMLparam1(unit);
   CAMLlocal1(block);
   shout_t *s = shout_new();
-  /*  150: beurk beurk beurk */
-  block = caml_alloc_final(2, finalize_shout_t, 150, 1000);
   if (s == NULL)
     caml_raise_constant(*caml_named_value("shout_exn_malloc"));
-  Store_field(block, 1, (value)s);
+  block = caml_alloc_custom(&shout_ops, sizeof(shout_t*), 0, 1);
+  Shout_val(block) = s;
   CAMLreturn(block);
 }
 
 CAMLprim value ocaml_shout_get_error(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   const char *error = shout_get_error(s);
   CAMLreturn(caml_copy_string(error));
 }
@@ -103,7 +109,7 @@ CAMLprim value ocaml_shout_get_error(value block)
 CAMLprim value ocaml_shout_get_errno(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int error = shout_get_errno(s);
   CAMLreturn(Val_int(error));
 }
@@ -111,7 +117,7 @@ CAMLprim value ocaml_shout_get_errno(value block)
 CAMLprim value ocaml_shout_get_connected(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   CAMLreturn(shout_get_connected(s)==SHOUTERR_CONNECTED?Val_true:Val_false);
 }
 
@@ -159,6 +165,7 @@ static void check_errors(int err)
       break;
 
     default:
+      assert(42 == 666);
       break;
     }
 }
@@ -172,7 +179,7 @@ static value unit_or_error(int err)
 CAMLprim value ocaml_shout_set_host(value block, value host)
 {
   CAMLparam2(block, host);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int ret = shout_set_host(s, String_val(host));
   CAMLreturn(unit_or_error(ret));
 }
@@ -180,7 +187,7 @@ CAMLprim value ocaml_shout_set_host(value block, value host)
 CAMLprim value ocaml_shout_get_host(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   const char *host = shout_get_host(s);
   CAMLreturn(caml_copy_string(host));
 }
@@ -188,7 +195,7 @@ CAMLprim value ocaml_shout_get_host(value block)
 CAMLprim value ocaml_shout_set_port(value block, value port)
 {
   CAMLparam2(block, port);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int ret = shout_set_port(s, (unsigned short)Int_val(port));
   CAMLreturn(unit_or_error(ret));
 }
@@ -196,7 +203,7 @@ CAMLprim value ocaml_shout_set_port(value block, value port)
 CAMLprim value ocaml_shout_get_port(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int port = shout_get_port(s);
   CAMLreturn(Val_int(port));
 }
@@ -204,7 +211,7 @@ CAMLprim value ocaml_shout_get_port(value block)
 CAMLprim value ocaml_shout_set_password(value block, value password)
 {
   CAMLparam2(block, password);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int ret = shout_set_password(s, String_val(password));
   CAMLreturn(unit_or_error(ret));
 }
@@ -212,7 +219,7 @@ CAMLprim value ocaml_shout_set_password(value block, value password)
 CAMLprim value ocaml_shout_get_password(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   const char *password = shout_get_password(s);
   CAMLreturn(caml_copy_string(password));
 }
@@ -220,7 +227,7 @@ CAMLprim value ocaml_shout_get_password(value block)
 CAMLprim value ocaml_shout_set_mount(value block, value mount)
 {
   CAMLparam2(block, mount);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int ret = shout_set_mount(s, String_val(mount));
   CAMLreturn(unit_or_error(ret));
 }
@@ -228,7 +235,7 @@ CAMLprim value ocaml_shout_set_mount(value block, value mount)
 CAMLprim value ocaml_shout_get_mount(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   const char *mount = shout_get_mount(s);
   CAMLreturn(caml_copy_string(mount));
 }
@@ -236,7 +243,7 @@ CAMLprim value ocaml_shout_get_mount(value block)
 CAMLprim value ocaml_shout_set_name(value block, value name)
 {
   CAMLparam2(block, name);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int ret = shout_set_name(s, String_val(name));
   CAMLreturn(unit_or_error(ret));
 }
@@ -244,7 +251,7 @@ CAMLprim value ocaml_shout_set_name(value block, value name)
 CAMLprim value ocaml_shout_get_name(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   const char *name = shout_get_name(s);
   CAMLreturn(caml_copy_string(name));
 }
@@ -252,7 +259,7 @@ CAMLprim value ocaml_shout_get_name(value block)
 CAMLprim value ocaml_shout_set_url(value block, value url)
 {
   CAMLparam2(block, url);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int ret = shout_set_url(s, String_val(url));
   CAMLreturn(unit_or_error(ret));
 }
@@ -260,7 +267,7 @@ CAMLprim value ocaml_shout_set_url(value block, value url)
 CAMLprim value ocaml_shout_get_url(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   const char *url = shout_get_url(s);
   CAMLreturn(caml_copy_string(url));
 }
@@ -268,7 +275,7 @@ CAMLprim value ocaml_shout_get_url(value block)
 CAMLprim value ocaml_shout_set_genre(value block, value genre)
 {
   CAMLparam2(block, genre);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int ret = shout_set_genre(s, String_val(genre));
   CAMLreturn(unit_or_error(ret));
 }
@@ -276,7 +283,7 @@ CAMLprim value ocaml_shout_set_genre(value block, value genre)
 CAMLprim value ocaml_shout_get_genre(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   const char *genre = shout_get_genre(s);
   CAMLreturn(caml_copy_string(genre));
 }
@@ -284,7 +291,7 @@ CAMLprim value ocaml_shout_get_genre(value block)
 CAMLprim value ocaml_shout_set_user(value block, value username)
 {
   CAMLparam2(block, username);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int ret = shout_set_user(s, String_val(username));
   CAMLreturn(unit_or_error(ret));
 }
@@ -292,7 +299,7 @@ CAMLprim value ocaml_shout_set_user(value block, value username)
 CAMLprim value ocaml_shout_get_user(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   const char *username = shout_get_user(s);
   CAMLreturn(caml_copy_string(username));
 }
@@ -300,7 +307,7 @@ CAMLprim value ocaml_shout_get_user(value block)
 CAMLprim value ocaml_shout_set_agent(value block, value username)
 {
   CAMLparam2(block, username);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int ret = shout_set_agent(s, String_val(username));
   CAMLreturn(unit_or_error(ret));
 }
@@ -308,7 +315,7 @@ CAMLprim value ocaml_shout_set_agent(value block, value username)
 CAMLprim value ocaml_shout_get_agent(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   const char *username = shout_get_agent(s);
   CAMLreturn(caml_copy_string(username));
 }
@@ -316,7 +323,7 @@ CAMLprim value ocaml_shout_get_agent(value block)
 CAMLprim value ocaml_shout_set_description(value block, value desription)
 {
   CAMLparam2(block, desription);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int ret = shout_set_description(s, String_val(desription));
   CAMLreturn(unit_or_error(ret));
 }
@@ -324,7 +331,7 @@ CAMLprim value ocaml_shout_set_description(value block, value desription)
 CAMLprim value ocaml_shout_get_description(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   const char *desription = shout_get_description(s);
   CAMLreturn(caml_copy_string(desription));
 }
@@ -332,7 +339,7 @@ CAMLprim value ocaml_shout_get_description(value block)
 CAMLprim value ocaml_shout_set_dumpfile(value block, value dumpfile)
 {
   CAMLparam2(block, dumpfile);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int ret = shout_set_dumpfile(s, String_val(dumpfile));
   CAMLreturn(unit_or_error(ret));
 }
@@ -340,7 +347,7 @@ CAMLprim value ocaml_shout_set_dumpfile(value block, value dumpfile)
 CAMLprim value ocaml_shout_get_dumpfile(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   const char *dumpfile = shout_get_dumpfile(s);
   CAMLreturn(caml_copy_string(dumpfile));
 }
@@ -348,7 +355,7 @@ CAMLprim value ocaml_shout_get_dumpfile(value block)
 CAMLprim value ocaml_shout_set_audio_info(value block, value name, value val)
 {
   CAMLparam3(block, name, val);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int ret = shout_set_audio_info(s, String_val(name), String_val(val));
   CAMLreturn(unit_or_error(ret));
 }
@@ -356,7 +363,7 @@ CAMLprim value ocaml_shout_set_audio_info(value block, value name, value val)
 CAMLprim value ocaml_shout_get_audio_info(value block, value name)
 {
   CAMLparam2(block, name);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   const char *val = shout_get_audio_info(s, String_val(name));
   CAMLreturn(caml_copy_string(val));
 }
@@ -364,7 +371,7 @@ CAMLprim value ocaml_shout_get_audio_info(value block, value name)
 CAMLprim value ocaml_shout_set_public(value block, value public)
 {
   CAMLparam2(block, public);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int ret = shout_set_public(s, (unsigned int)Int_val(public));
   CAMLreturn(unit_or_error(ret));
 }
@@ -372,7 +379,7 @@ CAMLprim value ocaml_shout_set_public(value block, value public)
 CAMLprim value ocaml_shout_get_public(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   unsigned int public = shout_get_public(s);
   CAMLreturn(public?Val_true:Val_false);
 }
@@ -382,7 +389,7 @@ static int data_format_table[] = {SHOUT_FORMAT_VORBIS, SHOUT_FORMAT_MP3};
 CAMLprim value ocaml_shout_set_format(value block, value data_format)
 {
   CAMLparam2(block, data_format);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int ret = shout_set_format(s, data_format_table[Int_val(data_format)]);
   CAMLreturn(unit_or_error(ret));
 }
@@ -390,7 +397,7 @@ CAMLprim value ocaml_shout_set_format(value block, value data_format)
 CAMLprim value ocaml_shout_get_format(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int data_format = shout_get_format(s);
   /* Not very clean: we should use data_format_table */
   CAMLreturn(Val_int(data_format));
@@ -401,7 +408,7 @@ static int protocol_table[] = {SHOUT_PROTOCOL_HTTP, SHOUT_PROTOCOL_XAUDIOCAST, S
 CAMLprim value ocaml_shout_set_protocol(value block, value protocol)
 {
   CAMLparam2(block, protocol);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int ret = shout_set_protocol(s, protocol_table[Int_val(protocol)]);
   CAMLreturn(unit_or_error(ret));
 }
@@ -409,7 +416,7 @@ CAMLprim value ocaml_shout_set_protocol(value block, value protocol)
 CAMLprim value ocaml_shout_get_protocol(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int protocol = shout_get_protocol(s);
   /* Not very clean: we should use protocol_table */
   CAMLreturn(Val_int(protocol));
@@ -418,7 +425,7 @@ CAMLprim value ocaml_shout_get_protocol(value block)
 CAMLprim value ocaml_shout_open(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int ret = shout_open(s);
   CAMLreturn(unit_or_error(ret));
 }
@@ -426,7 +433,7 @@ CAMLprim value ocaml_shout_open(value block)
 CAMLprim value ocaml_shout_close(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int ret = shout_close(s);
   CAMLreturn(unit_or_error(ret));
 }
@@ -434,7 +441,7 @@ CAMLprim value ocaml_shout_close(value block)
 CAMLprim value ocaml_shout_send(value block, value data)
 {
   CAMLparam2(block, data);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   size_t len = caml_string_length(data);
   unsigned char* dat = malloc(len);
   int ret;
@@ -451,7 +458,7 @@ CAMLprim value ocaml_shout_send(value block, value data)
 CAMLprim value ocaml_shout_send_raw(value block, value data)
 {
   CAMLparam2(block, data);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   size_t len = caml_string_length(data);
   unsigned char* dat = malloc(len);
   int ret;
@@ -468,7 +475,7 @@ CAMLprim value ocaml_shout_send_raw(value block, value data)
 CAMLprim value ocaml_shout_sync(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   caml_enter_blocking_section();
   shout_sync(s);
   caml_leave_blocking_section();
@@ -478,7 +485,7 @@ CAMLprim value ocaml_shout_sync(value block)
 CAMLprim value ocaml_shout_delay(value block)
 {
   CAMLparam1(block);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   int delay = shout_delay(s);
   CAMLreturn(Val_int(delay));
 }
@@ -486,7 +493,7 @@ CAMLprim value ocaml_shout_delay(value block)
 CAMLprim value ocaml_shout_set_metadata(value block, value data)
 {
   CAMLparam2(block, data);
-  shout_t *s = shout_of_block(block);
+  shout_t *s = Shout_val(block);
   shout_metadata_t *metadata = shout_metadata_new();
   int i, ret;
   char *name, *val;
